#include "GUIManager.h"
#include <iostream>

using namespace CEGUI;

// Xg{bNX̍
static char *FontList [] = 
{
    "j[O",
    "j[P",
    "j[Q",
    "j[R",
    "j[S"
};

#define MIN_POINT_SIZE 6.0f 

// Sample sub-class for ListboxTextItem that auto-sets the selection brush
// image.  This saves doing it manually every time in the code.
class MyListItem : public ListboxTextItem
{
public:
    MyListItem (const String& text) : ListboxTextItem(text)
    {
        setSelectionBrushImage("TaharezLook", "MultiListSelectionBrush");
    }
}; 

CGUIManager::CGUIManager(void)
{
    for (int i=0; i<6; i++)
    {
        data[i] = 0;
    }

    // CEGUIOpenGLRenderer쐬
    renderer = new CEGUI::OpenGLRenderer(0, 800, 600);
    // CEGUISystem쐬
    system = new CEGUI::System(renderer);
    

    // Ox̐ݒ
    CEGUI::Logger::getSingleton().setLoggingLevel(CEGUI::Informative);

    // TaharezLook̃XLI
    CEGUI::SchemeManager::getSingleton().loadScheme("TaharezLook.scheme");
    // }EXJ[\̐ݒ
    system->setDefaultMouseCursor((CEGUI::utf8*)"TaharezLook", "MouseArrow");

    // tHg̐ݒ
    CEGUI::FontManager::getSingleton().createFont("sazanami-gothic.font");
    system->setDefaultFont((CEGUI::utf8*)"sazanami-gothic");
    system->getDefaultFont()->setProperty("PointSize", "9");

    // CAEg̓ǂݍ݁EݒiCEGUI Layout Editorō쐬j
    layout = CEGUI::WindowManager::getSingleton().loadWindowLayout("gluttest.layout");
    system->setGUISheet(layout);

    d_lastFrameTime = 0;

    initGUI(this);
}

void CGUIManager::initGUI(CGUIManager *pointer)
{
    // Xg{bNXւ̃|C^̎擾
    CEGUI::Listbox *lbox = static_cast<Listbox *>(CEGUI::WindowManager::getSingleton().getWindow("Root/FW01/LT01"));
    char str[256];
    int str_size=256;
    char *pSjisMes;

    if (lbox != NULL)
    {
        // Xg{bNXɃXgǉ
        for (size_t i = 0; i < (sizeof (FontList) / sizeof (FontList [0])); i++)
        {
            sprintf_s(str, FontList[i]);
            pSjisMes = CUnicodeF::sjis_to_utf8(str, &str_size);
            lbox->addItem (new MyListItem ((utf8*)pSjisMes));
        }

#if 0
        // set up the language listbox callback
        lbox->subscribeEvent(Listbox::EventListContentsChanged, CEGUI::Event::Subscriber(&CGUIManager::OnListClick, this));
#endif
        // 1Ԗڂ̃XgIԂɐݒ
        lbox->setItemSelectState (size_t (0), true);
    }

    // GfBbg{bNX̒l
    CEGUI::Editbox *edit;
    for (int i=0; i<6; i++)
    {
        sprintf_s(str, 256, "Root/FW03/EB0%d", i+1);
        printf("%s\n", str);
        edit = static_cast<CEGUI::Editbox *>(CEGUI::WindowManager::getSingleton().getWindow(str));
        if (edit != NULL)
        {
            edit->setText("0");
        }
    }

}

CGUIManager::~CGUIManager(void)
{
    if (system != NULL)
    {
        delete system;
    }

    if (renderer != NULL)
    {
        delete renderer;
    }
}

void CGUIManager::checkEvent()
{
    // Xg{bNX̃Cxg`FbN
    size_t index = -1;
    CEGUI::Listbox *lbox = static_cast<Listbox *>(CEGUI::WindowManager::getSingleton().getWindow("Root/FW01/LT01"));
    for (size_t i=0; i<lbox->getItemCount(); i++)
    {
        if (lbox->isItemSelected(i))
        {
            index = i;
        }
    }

    // \̍쐬
    char str[256];
    int str_size=256;
    sprintf_s(str, "IĂ郁j[%dłB", index);

    // \
    setDebugText(str);

    // {^̃Cxg`FbN
    CEGUI::PushButton *button = static_cast<PushButton *>(CEGUI::WindowManager::getSingleton().getWindow("Root/BT01"));
    // Exit{^ꂽꍇ͏I
    if (button->isPushed())
    {
        exit(0);
    }

    button  = static_cast<PushButton *>(CEGUI::WindowManager::getSingleton().getWindow("Root/FW03/BT01"));
    // Update{^ꂽꍇ͒l擾
    if (button->isPushed())
    {
        // GfBbg{bNX̒l擾
        CEGUI::Editbox *edit;
        for (int i=0; i<6; i++)
        {
            sprintf_s(str, 256, "Root/FW03/EB0%d", i+1);
            edit = static_cast<CEGUI::Editbox *>(CEGUI::WindowManager::getSingleton().getWindow(str));

            data[i] = atof(edit->getText().c_str());
        }
    }

}

// VXe݂Ε`
void CGUIManager::renderGUI()
{
    if (system != NULL)
    {
        // subscribeEvent֐łȂsɗ̂ŎŃCxg`FbN
        checkEvent();
        int thisTime = glutGet(GLUT_ELAPSED_TIME);
        float elapsed = static_cast<float>(thisTime - d_lastFrameTime);
        d_lastFrameTime = thisTime;
        // inject the time pulse
        system->injectTimePulse(elapsed / 1000.0f);

        system->renderGUI();
    }
}

// EBhEɃeLXg\
void CGUIManager::setDebugText(const char* text)
{
    int textsize = sizeof(text);
    char *pSjisMes = CUnicodeF::sjis_to_utf8(text, &textsize);
    CEGUI::Window *debug = static_cast<Window*> (CEGUI::WindowManager::getSingleton().getWindow("Root/FW02/ST01"));
    debug->setText((utf8*)pSjisMes);
}

void CGUIManager::fakeCtrlAltShift()
{
    int status = glutGetModifiers();
    if (status & GLUT_ACTIVE_CTRL)
    {
        if (!ctrl_held)
        {
            ctrl_held = true;
            CEGUI::System::getSingleton().injectKeyDown(CEGUI::Key::LeftControl);
        }
    }
    else
    {
        if (ctrl_held)
        {
            ctrl_held = false;
            CEGUI::System::getSingleton().injectKeyUp(CEGUI::Key::LeftControl);
        }
    }
    //
    if (status & GLUT_ACTIVE_ALT)
    {
        if (!alt_held)
        {
            alt_held = true;
            CEGUI::System::getSingleton().injectKeyDown(CEGUI::Key::LeftAlt);
        }
    }
    else
    {
        if (alt_held)
        {
            alt_held = false;
            CEGUI::System::getSingleton().injectKeyUp(CEGUI::Key::LeftAlt);
        }
    }
    //
    if (status & GLUT_ACTIVE_SHIFT)
    {
        if (!shift_held)
        {
            shift_held = true;
            CEGUI::System::getSingleton().injectKeyDown(CEGUI::Key::LeftShift);
        }
    }
    else
    {
        if (shift_held)
        {
            shift_held = false;
            CEGUI::System::getSingleton().injectKeyUp(CEGUI::Key::LeftShift);
        }
    }
}

// Map ASCII to Scan Codes
int CGUIManager::mapNormal(unsigned char c)
{
    int scancode;
    switch(c)
    {
    case 97:
    case 65:
        scancode = CEGUI::Key::A;
        break;
    case 98:
    case 66:
        scancode = CEGUI::Key::B;
        break;
    case 99:
    case 67:
        scancode = CEGUI::Key::C;
        break;
    case 100:
    case 68:
        scancode = CEGUI::Key::D;
        break;
    case 101:
    case 69:
        scancode = CEGUI::Key::E;
        break;
    case 102:
    case 70:
        scancode = CEGUI::Key::F;
        break;
    case 103:
    case 71:
        scancode = CEGUI::Key::G;
        break;
    case 104:
    case 72:
        scancode = CEGUI::Key::H;
        break;
    case 105:
    case 73:
        scancode = CEGUI::Key::I;
        break;
    case 106:
    case 74:
        scancode = CEGUI::Key::J;
        break;
    case 107:
    case 75:
        scancode = CEGUI::Key::K;
        break;
    case 108:
    case 76:
        scancode = CEGUI::Key::L;
        break;
    case 109:
    case 77:
        scancode = CEGUI::Key::M;
        break;
    case 110:
    case 78:
        scancode = CEGUI::Key::N;
        break;
    case 111:
    case 79:
        scancode = CEGUI::Key::O;
        break;
    case 112:
    case 80:
        scancode = CEGUI::Key::P;
        break;
    case 113:
    case 81:
        scancode = CEGUI::Key::Q;
        break;
    case 114:
    case 82:
        scancode = CEGUI::Key::R;
        break;
    case 115:
    case 83:
        scancode = CEGUI::Key::S;
        break;
    case 116:
    case 84:
        scancode = CEGUI::Key::T;
        break;
    case 117:
    case 85:
        scancode = CEGUI::Key::U;
        break;
    case 118:
    case 86:
        scancode = CEGUI::Key::V;
        break;
    case 119:
    case 87:
        scancode = CEGUI::Key::W;
        break;
    case 120:
    case 88:
        scancode = CEGUI::Key::X;
        break;
    case 121:
    case 89:
        scancode = CEGUI::Key::Y;
        break;
    case 122:
    case 90:
        scancode = CEGUI::Key::Z;
        break;
    case 27:
        scancode = CEGUI::Key::Escape;
        break;
    case 49:
        scancode = CEGUI::Key::One;
        break;
    case 50:
        scancode = CEGUI::Key::Two;
        break;
    case 51:
        scancode = CEGUI::Key::Three;
        break;
    case 52:
        scancode = CEGUI::Key::Four;
        break;
    case 53:
        scancode = CEGUI::Key::Five;
        break;
    case 54:
        scancode = CEGUI::Key::Six;
        break;
    case 55:
        scancode = CEGUI::Key::Seven;
        break;
    case 56:
        scancode = CEGUI::Key::Eight;
        break;
    case 57:
        scancode = CEGUI::Key::Nine;
        break;
    case 48:
        scancode = CEGUI::Key::Zero;
        break;
    case 45:
        scancode = CEGUI::Key::Minus;
        break;
    case 61:
        scancode = CEGUI::Key::Equals;
        break;
    case 8:
        scancode = CEGUI::Key::Backspace;
        break;
    case 9:
        scancode = CEGUI::Key::Tab;
        break;
    case 91:
        scancode = CEGUI::Key::LeftBracket;
        break;
    case 93:
        scancode = CEGUI::Key::RightBracket;
        break;
    case 13:
        scancode = CEGUI::Key::Return;
        break;
    case 59:
        scancode = CEGUI::Key::Semicolon;
        break;
    case 39:
        scancode = CEGUI::Key::Apostrophe;
        break;
    case 96:
        scancode = CEGUI::Key::Grave;
        break;
    case 92:
        scancode = CEGUI::Key::Backslash;
        break;
    case 44:
        scancode = CEGUI::Key::Comma;
        break;
    case 46:
        scancode = CEGUI::Key::Period;
        break;
    case 47:
        scancode = CEGUI::Key::Slash;
        break;
    case 42:
        scancode = CEGUI::Key::Multiply;
        break;
    case 32:
        scancode = CEGUI::Key::Space;
        break;
    case 64:
        scancode = CEGUI::Key::At;
        break;
    case 58:
        scancode = CEGUI::Key::Colon;
        break;
    case 95:
        scancode = CEGUI::Key::Underline;
        break;
    case 127:
        scancode = CEGUI::Key::Delete;
        break;
    default:
        scancode = 0;
    }
    return scancode;
}

// Map Special GLUT Keys to Scan Codes
int CGUIManager::mapSpecial(int c)
{
    int scancode;
    switch(c)
    {
    case 1:
        scancode = CEGUI::Key::F1;
        break;
    case 2:
        scancode = CEGUI::Key::F2;
        break;
    case 3:
        scancode = CEGUI::Key::F3;
        break;
    case 4:
        scancode = CEGUI::Key::F4;
        break;
    case 5:
        scancode = CEGUI::Key::F5;
        break;
    case 6:
        scancode = CEGUI::Key::F6;
        break;
    case 7:
        scancode = CEGUI::Key::F7;
        break;
    case 8:
        scancode = CEGUI::Key::F8;
        break;
    case 9:
        scancode = CEGUI::Key::F9;
        break;
    case 10:
        scancode = CEGUI::Key::F10;
        break;
    case 11:
        scancode = CEGUI::Key::F11;
        break;
    case 12:
        scancode = CEGUI::Key::F12;
        break;
    case 104:
        scancode = CEGUI::Key::PageUp;
        break;
    case 105:
        scancode = CEGUI::Key::PageDown;
        break;
    case 106:
        scancode = CEGUI::Key::Home;
        break;
    case 107:
        scancode = CEGUI::Key::End;
        break;
    case 100:
        scancode = CEGUI::Key::ArrowLeft;
        break;
    case 101:
        scancode = CEGUI::Key::ArrowUp;
        break;
    case 102:
        scancode = CEGUI::Key::ArrowRight;
        break;
    case 103:
        scancode = CEGUI::Key::ArrowDown;
        break;
    case 108:
        scancode = CEGUI::Key::Insert;
        break;
    default:
        scancode = 0;
    }
    return scancode;
}

void CGUIManager::myKeyboardFunc(unsigned char key, int x, int y)
{
    fakeCtrlAltShift();
    CEGUI::System::getSingleton().injectChar(key);
    int scancode = mapNormal(key);
    if (scancode)
    {
        CEGUI::System::getSingleton().injectKeyUp(scancode);
    }
}

void CGUIManager::myKeyboardUpFunc(unsigned char key, int x, int y)
{
    fakeCtrlAltShift();
    int scancode = mapNormal(key);
    if (scancode)
    {
        CEGUI::System::getSingleton().injectKeyUp(scancode);
    }
}

void CGUIManager::mySpecialFunc(int key, int x, int y)
{
    fakeCtrlAltShift();
    int scancode = mapSpecial(key);
    if (scancode)
    {
        CEGUI::System::getSingleton().injectKeyDown(scancode);
    }
}

void CGUIManager::mySpecialUpFunc(int key, int x, int y)
{
    fakeCtrlAltShift();
    int scancode = mapSpecial(key);
    if (scancode)
    {
        CEGUI::System::getSingleton().injectKeyUp(scancode);
    }
}

void CGUIManager::myMouseFunc(int button, int state, int x, int y)
{
    if (state == 0)     // State 0 = Button Pressed
    {
        switch (button)
        {
        case 0:     // glut's left mouse button
            CEGUI::System::getSingleton().injectMouseButtonDown(CEGUI::LeftButton);
            break;
        case 1:     // glut's middle mouse button
            CEGUI::System::getSingleton().injectMouseButtonDown(CEGUI::MiddleButton);
            break;
        case 2:     // glut's right mouse button
            CEGUI::System::getSingleton().injectMouseButtonDown(CEGUI::RightButton);
            break;
        case 3:     // glut's mouse wheel up
            CEGUI::System::getSingleton().injectMouseWheelChange(2.0);
            break;
        case 4:     // glut's mouse wheen down
            CEGUI::System::getSingleton().injectMouseWheelChange(-2.0);
            break;
        }
    }
    else if (state == 1)    // State 1 = Button Released
    {
        switch (button)
        {
        case 0:     // glut's left mouse button
            CEGUI::System::getSingleton().injectMouseButtonUp(CEGUI::LeftButton);
            break;
        case 1:     // glut's middle mouse button
            CEGUI::System::getSingleton().injectMouseButtonUp(CEGUI::MiddleButton);
            break;
        case 2:     // glut's right mouse button
            CEGUI::System::getSingleton().injectMouseButtonUp(CEGUI::RightButton);
            break;
        }
    }
}

void CGUIManager::myPassiveMotionFunc(int x, int y)
{
    CEGUI::System::getSingleton().injectMousePosition(x,y);
}

void CGUIManager::myMotionFunc(int x, int y)
{
    CEGUI::System::getSingleton().injectMousePosition(x,y);
}

bool CGUIManager::OnListClick(const CEGUI::EventArgs &args)
{
    // NbÑCxg烊Xg{bNX̃|C^֕ϊ
    Listbox *lbox = static_cast<Listbox *> (
        static_cast<const WindowEventArgs&> (args).window);
 
    // NbNItemindex擾
    size_t idx;
    if (lbox->getFirstSelectedItem ())
    {
        idx = lbox->getItemIndex (lbox->getFirstSelectedItem ());
    }
 
    // \̍쐬
    char str[256];
    int str_size=256;
    sprintf_s(str, "IĂ郁j[%dłB", idx);
    // \
    setDebugText(str);
 
    return true;
}