#include "GUIManager.h"

CGUIManager *cegui;

GLfloat mat_ambient[]   = { 0.7, 0.5, 0.5, 1.0 },
        mat_diffuse[]   = { 0.1, 0.5, 0.8, 1.0 },
        mat_specular[]  = { 1.0, 1.0, 1.0, 1.0 },
        mat_shininess[] = { 100.0 };

/* _ƒ_̃f[^ */
static GLdouble eye[3] = { 0.0, 0.0, 5.0 };
static GLdouble center[3] = { 0.0, 0.0, 0.0 };
static GLdouble up[3] = { 0.0, 1.0, 0.0 }; /* r[AbvxNg */

/* Ɩ̈ʒu */
static GLfloat light_position0[] = { 0.0, 0.0, 1.0, 0.0 };
static GLfloat light_position1[] = { 50.0, 0.0, 50.0, 1.0 };

/* Ɩ̐ݒ */
void initLights(void)
{
    GLfloat ambient[] = { 0.2, 0.2, 0.2, 1.0 };
    GLfloat diffuse[] = { 0.1, 0.8, 0.8, 1.0 };
    GLfloat specular[] = { 1.0, 1.0, 1.0, 1.0 };
    GLfloat shininess[] = { 50.0 };
    GLfloat light_param[] = { 0.5, 0.5, 0.5, 1.0 };

    glLightfv ( GL_LIGHT0, GL_AMBIENT, ambient );
    glLightfv ( GL_LIGHT0, GL_POSITION, light_position0 );
    glLightfv ( GL_LIGHT0, GL_DIFFUSE, light_param );
    glLightfv ( GL_LIGHT1, GL_DIFFUSE, light_param );
    glLightfv ( GL_LIGHT1, GL_SPECULAR, light_param );
    glMaterialfv ( GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse );
    glMaterialfv ( GL_FRONT_AND_BACK, GL_SPECULAR, specular );
    glMaterialfv ( GL_FRONT_AND_BACK, GL_SHININESS, shininess );

    glEnable ( GL_LIGHTING );
    glEnable ( GL_LIGHT0 ); 
    glEnable ( GL_LIGHT1 );
}

//
// This is where you'd draw a frame of your 3D application
//
void render()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
    glEnable(GL_DEPTH_TEST);
    glEnable ( GL_LIGHTING );

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    gluLookAt(eye[0], eye[1], eye[2], center[0], center[1], center[2],
              up[0], up[1], up[2]);

    glPushMatrix();
    glLightfv ( GL_LIGHT1, GL_POSITION, light_position1 );
    glPopMatrix();

    glPushMatrix();
	// ގݒ
    glMaterialfv(GL_FRONT, GL_AMBIENT,  mat_ambient); 
    glMaterialfv(GL_FRONT, GL_DIFFUSE,  mat_diffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
    glMaterialfv(GL_FRONT, GL_SHININESS,mat_shininess);
    glTranslatef(cegui->data[0], cegui->data[1], cegui->data[2]);
    glutSolidTeapot( 1.0 );
    glPopMatrix();

    glDisable(GL_LIGHTING);
}

//
//  Handler for normal keypressed
//
void keyFunc(unsigned char key, int x, int y)
{
    cegui->myKeyboardFunc(key, x, y);

    switch (key)
    {
    case 113:   // 'q',
    case 81:    // 'Q',
    case 27:    // 'ESC'
        exit(0);
        break;
    }
}

void init()
{
    glClearColor(0.4f, 0.4f, 0.7f,1.0f);

    initLights();

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_AUTO_NORMAL);
    glEnable(GL_NORMALIZE);
	glShadeModel(GL_SMOOTH);
	glDepthFunc(GL_LEQUAL);

    glEnable(GL_LINE_SMOOTH);
    glEnable(GL_POLYGON_SMOOTH);

    glDisable(GL_FOG);
}

void reshape(int w, int h)
{
    glViewport(0,0, 800,600);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0, 800.0/600.0, 0.1,100.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void idle()
{
    render();
    cegui->renderGUI();
    glutSwapBuffers();
}

void release()
{
    if (cegui != NULL)
    {
        delete cegui;
    }
}

void keyUpFunc(unsigned char key, int x, int y)
{
    cegui->myKeyboardUpFunc(key, x, y);

}

void specialFunc(int key, int x, int y)
{
    cegui->mySpecialFunc(key, x, y);
}

void specialUpFunc(int key, int x, int y)
{
    cegui->mySpecialUpFunc(key, x, y);
}

void mouseFunc(int button, int state, int x, int y)
{
    cegui->myMouseFunc(button, state, x, y);
}

void passiveMotionFunc(int x, int y)
{
    cegui->myPassiveMotionFunc(x, y);
}

void motionFunc(int x, int y)
{
    cegui->myMotionFunc(x, y);
}

int main(int argc, char *argv[])
{
    glutInit(&argc, argv);
    glutInitWindowPosition(100, 100);
    glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
    glutInitWindowSize(800, 600);
    glutCreateWindow("GLUT+CEGUI Test");
    glutReshapeFunc(reshape);
    init();
    glutSetCursor(GLUT_CURSOR_NONE);

    cegui = new CGUIManager();

    // Set some input handlers
    glutKeyboardFunc(keyFunc);              // key pressed
    glutKeyboardUpFunc(keyUpFunc);          // key released
    glutSpecialFunc(specialFunc);                // special key pressed
    glutSpecialUpFunc(specialUpFunc);            // special key released
    glutMouseFunc(mouseFunc);                    // any mouse button press or release
    glutPassiveMotionFunc(passiveMotionFunc);    // mouse movement with no buttons held
    glutMotionFunc(motionFunc);                  // mouse movement with any button held
    glutDisplayFunc(render);
    glutIdleFunc(idle);

    atexit(release);

    glutMainLoop();

    return 0;
}